@@ -1,6 +1,7 @@
 # For details on how netCDF4 builds on HDF5:
 # http://www.unidata.ucar.edu/software/netcdf/docs/file_format_specifications.html#netcdf_4_spec
-from collections import Mapping
+from collections import defaultdict
+from collections.abc import Mapping
 import os.path
 import warnings
 
@@ -23,7 +24,7 @@ else:
     no_h5pyd = False
     h5_group_types = (h5py.Group, h5pyd.Group)
 
-__version__ = '0.7.1'
+__version__ = '0.8.1'
 
 
 _NC_PROPERTIES = (u'version=2,h5netcdf=%s,hdf5=%s,h5py=%s'
@@ -95,16 +96,25 @@ class BaseVariable(object):
             return (child_name,)
 
         dims = []
+        phony_dims = defaultdict(int)
         for axis, dim in enumerate(self._h5ds.dims):
-            # TODO: read dimension labels even if there is no associated
-            # scale? it's not netCDF4 spec, but it is unambiguous...
-            # Also: the netCDF lib can read HDF5 datasets with unlabeled
-            # dimensions.
-            if len(dim) == 0:
-                raise ValueError('variable %r has no dimension scale '
-                                 'associated with axis %s'
-                                 % (self.name, axis))
-            name = _name_from_dimension(dim)
+            # get current dimension
+            dimsize = self.shape[axis]
+            phony_dims[dimsize] += 1
+            if len(dim):
+                name = _name_from_dimension(dim)
+            else:
+                # if unlabeled dimensions are found
+                if self._root._phony_dims_mode is None:
+                    raise ValueError('variable %r has no dimension scale '
+                                     'associated with axis %s. \n'
+                                     'Use phony_dims=%r for sorted naming or '
+                                     'phony_dims=%r for per access naming.'
+                                     % (self.name, axis, 'sort', 'access'))
+                else:
+                    # get dimension name
+                    name = self._parent._phony_dims[(dimsize,
+                                                     phony_dims[dimsize] - 1)]
             dims.append(name)
         return tuple(dims)
 
@@ -211,6 +221,16 @@ def _netcdf_dimension_but_not_variable(h5py_dataset):
     return NOT_A_VARIABLE in h5py_dataset.attrs.get('NAME', b'')
 
 
+def _unlabeled_dimension_mix(h5py_dataset):
+    dims = sum([len(j) for j in h5py_dataset.dims])
+    if dims:
+        if dims != h5py_dataset.ndim:
+            name = h5py_dataset.name.split('/')[-1]
+            raise ValueError('malformed variable %r has mixing of labeled and '
+                             'unlabeled dimensions.'.format(name))
+    return dims
+
+
 class Group(Mapping):
 
     _variable_cls = Variable
@@ -233,6 +253,12 @@ class Group(Mapping):
         self._variables = _LazyObjectLookup(self, self._variable_cls)
         self._groups = _LazyObjectLookup(self, self._group_cls)
 
+        # # initialize phony dimension counter
+        if self._root._phony_dims_mode is not None:
+            self._phony_dims = {}
+            phony_dims = defaultdict(int)
+            labeled_dims = defaultdict(int)
+
         for k, v in self._h5group.items():
             if isinstance(v, h5_group_types):
                 # add to the groups collection if this is a h5py(d) Group
@@ -253,23 +279,65 @@ class Group(Mapping):
                         current_size = v.size
 
                     self._dim_sizes[k] = size
+
+                    # keep track of found labeled dimensions
+                    if self._root._phony_dims_mode is not None:
+                        labeled_dims[size] += 1
+                        self._phony_dims[(size, labeled_dims[size] - 1)] = k
+
                     # Figure out the current size of a dimension, which for
                     # unlimited dimensions requires looking at the actual
                     # variables.
                     self._current_dim_sizes[k] = \
                         self._determine_current_dimension_size(k, current_size)
 
-                    if dim_id is None:
-                        dim_id = self._root._get_next_dim_id()
                     self._dim_order[k] = dim_id
+                else:
+                    if self._root._phony_dims_mode is not None:
+                        # check if malformed variable
+                        if not _unlabeled_dimension_mix(v):
+                            # if unscaled variable, get phony dimensions
+                            vdims = defaultdict(int)
+                            for i in v.shape:
+                                vdims[i] += 1
+                            for dimsize, cnt in vdims.items():
+                                phony_dims[dimsize] = max(phony_dims[dimsize],
+                                                          cnt)
+
                 if not _netcdf_dimension_but_not_variable(v):
-                    var_name = k
-                    if k.startswith('_nc4_non_coord_'):
-                        var_name = k[len('_nc4_non_coord_'):]
-                    self._variables.add(var_name)
+                    if isinstance(v, h5py.Dataset):
+                        var_name = k
+                        if k.startswith('_nc4_non_coord_'):
+                            var_name = k[len('_nc4_non_coord_'):]
+                        self._variables.add(var_name)
+
+        # iterate over found phony dimensions and create them
+        if self._root._phony_dims_mode is not None:
+            grp_phony_count = 0
+            for size, cnt in phony_dims.items():
+                # only create missing dimensions
+                for pcnt in range(labeled_dims[size], cnt):
+                    name = (grp_phony_count +
+                            self._root._phony_dim_count)
+                    grp_phony_count += 1
+                    if self._root._phony_dims_mode == "access":
+                        name = "phony_dim_{}".format(name)
+                        self._create_dimension(name, size)
+                    self._phony_dims[(size, pcnt)] = name
+            # finally increase phony dim count at file level
+            self._root._phony_dim_count += grp_phony_count
 
         self._initialized = True
 
+    def _create_phony_dimensions(self):
+        # this is for 'sort' naming
+        for key, value in self._phony_dims.items():
+            if isinstance(value, int):
+                value += self._root._labeled_dim_count
+                name = "phony_dim_{}".format(value)
+                self._create_dimension(name, key[0])
+                self._phony_dims[key] = name
+
     def _determine_current_dimension_size(self, dim_name, max_size):
         """
         Helper method to determine the current size of a dimension.
@@ -315,7 +383,7 @@ class Group(Mapping):
 
         self._dim_sizes[name] = size
         self._current_dim_sizes[name] = 0 if size is None else size
-        self._dim_order[name] = self._root._get_next_dim_id()
+        self._dim_order[name] = None
 
     @property
     def dimensions(self):
@@ -476,8 +544,14 @@ class Group(Mapping):
                 coord_ids = np.array([dim_order[d] for d in dims], 'int32')
                 h5ds.attrs['_Netcdf4Coordinates'] = coord_ids
 
+            # TODO: don't re-create scales if they already exist. With the
+            # current version of h5py, this would require using the low-level
+            # h5py.h5ds.is_scale interface to detect pre-existing scales.
             scale_name = dim if dim in self.variables else NOT_A_VARIABLE
-            h5ds.dims.create_scale(h5ds, scale_name)
+            if h5py.__version__ < LooseVersion('2.10.0'):
+                h5ds.dims.create_scale(h5ds, scale_name)
+            else:
+                h5ds.make_scale(scale_name)
 
         for subgroup in self.groups.values():
             subgroup._create_dim_scales()
@@ -577,7 +651,8 @@ class Group(Mapping):
 
 class File(Group):
 
-    def __init__(self, path, mode='a', invalid_netcdf=None, **kwargs):
+    def __init__(self, path, mode='a', invalid_netcdf=None, phony_dims=None,
+                 **kwargs):
         try:
             if isinstance(path, str):
                 if path.startswith(('http://', 'https://', 'hdf5://')):
@@ -617,6 +692,16 @@ class File(Group):
         # if we actually use invalid NetCDF features.
         self._write_ncproperties = invalid_netcdf is not True
 
+        # phony dimension handling
+        self._phony_dims_mode = phony_dims
+        if phony_dims is not None:
+            self._phony_dim_count = 0
+            if phony_dims not in ['sort', 'access']:
+                raise ValueError('unknown value %r for phony_dims\n'
+                                 'Use phony_dims=%r for sorted naming, '
+                                 'phony_dims=%r for per access naming.'
+                                 % (phony_dims, 'sort', 'access'))
+
         # These maps keep track of dimensions in terms of size (might be
         # unlimited), current size (identical to size for limited dimensions),
         # their position, and look-up for HDF5 datasets corresponding to a
@@ -625,16 +710,26 @@ class File(Group):
         self._current_dim_sizes = ChainMap()
         self._dim_order = ChainMap()
         self._all_h5groups = ChainMap(self._h5group)
-
-        # used for picking numbers to use in self._dim_order
-        self._next_dim_id = 0
-
         super(File, self).__init__(self, self._h5path)
+        # initialize all groups to detect/create phony dimensions
+        # mimics netcdf-c style naming
+        if phony_dims == 'sort':
+            self._determine_phony_dimensions()
 
-    def _get_next_dim_id(self):
-        dim_id = self._next_dim_id
-        self._next_dim_id += 1
-        return dim_id
+    def _determine_phony_dimensions(self):
+        def get_labeled_dimension_count(grp):
+            count = len(grp._dim_sizes.maps[0])
+            for name in grp.groups:
+                count += get_labeled_dimension_count(grp[name])
+            return count
+
+        def create_phony_dimensions(grp):
+            grp._create_phony_dimensions()
+            for name in grp.groups:
+                create_phony_dimensions(grp[name])
+
+        self._labeled_dim_count = get_labeled_dimension_count(self)
+        create_phony_dimensions(self)
 
     def _check_valid_netcdf_dtype(self, dtype, stacklevel=3):
         dtype = np.dtype(dtype)
@@ -670,8 +765,32 @@ class File(Group):
     def parent(self):
         return None
 
+    def _set_unassigned_dimension_ids(self):
+        max_dim_id = -1
+
+        # collect the largest assigned dimension ID
+        groups = [self]
+        while groups:
+            group = groups.pop()
+            assigned_dim_ids = [dim_id for dim_id in group._dim_order.values()
+                                if dim_id is not None]
+            max_dim_id = max([max_dim_id] + assigned_dim_ids)
+            groups.extend(group._groups.values())
+
+        # set all dimension IDs to valid values
+        next_dim_id = max_dim_id + 1
+        groups = [self]
+        while groups:
+            group = groups.pop()
+            for key in group._dim_order:
+                if group._dim_order[key] is None:
+                    group._dim_order[key] = next_dim_id
+                    next_dim_id += 1
+            groups.extend(group._groups.values())
+
     def flush(self):
         if 'r' not in self._mode:
+            self._set_unassigned_dimension_ids()
             self._create_dim_scales()
             self._attach_dim_scales()
             if not self._preexisting_file and self._write_ncproperties:
