@@ -36,7 +36,7 @@ def tmp_local_netcdf(tmpdir):
 @pytest.fixture(params=['testfile.nc', 'hdf5://testfile'])
 def tmp_local_or_remote_netcdf(request, tmpdir):
     if request.param.startswith(remote_h5):
-        if not pytest.config.option.restapi:
+        if not request.config.option.restapi:
             pytest.skip('Do not test with HDF5 REST API')
         elif without_h5pyd:
             pytest.skip('h5pyd package not available')
@@ -482,13 +482,17 @@ def test_attrs_api(tmp_local_or_remote_netcdf):
 
 def test_optional_netcdf4_attrs(tmp_local_or_remote_netcdf):
     h5 = get_hdf5_module(tmp_local_or_remote_netcdf)
-    with h5.File(tmp_local_or_remote_netcdf) as f:
+    with h5.File(tmp_local_or_remote_netcdf, 'w') as f:
         foo_data = np.arange(50).reshape(5, 10)
         f.create_dataset('foo', data=foo_data)
         f.create_dataset('x', data=np.arange(5))
         f.create_dataset('y', data=np.arange(10))
-        f['foo'].dims.create_scale(f['x'])
-        f['foo'].dims.create_scale(f['y'])
+        if h5py.__version__ < LooseVersion('2.10.0'):
+            f['foo'].dims.create_scale(f['x'])
+            f['foo'].dims.create_scale(f['y'])
+        else:
+            f['x'].make_scale()
+            f['y'].make_scale()
         f['foo'].dims[0].attach_scale(f['x'])
         f['foo'].dims[1].attach_scale(f['y'])
     with h5netcdf.File(tmp_local_or_remote_netcdf, 'r') as ds:
@@ -514,15 +518,165 @@ def test_error_handling(tmp_local_or_remote_netcdf):
             ds.create_group('subgroup')
 
 
+def create_invalid_netcdf_data():
+    foo_data = np.arange(125).reshape(5, 5, 5)
+    bar_data = np.arange(625).reshape(25, 5, 5)
+    var = {'foo1': foo_data, 'foo2': bar_data,
+           'foo3': foo_data, 'foo4': bar_data}
+    var2 = {'x': 5, 'y': 5, 'z': 5, 'x1': 25, 'y1': 5, 'z1': 5}
+    return var, var2
+
+
+def check_invalid_netcdf4(var, i):
+    pdim = 'phony_dim_{}'
+    assert var['foo1'].dimensions[0] == pdim.format(i * 4)
+    assert var['foo1'].dimensions[1] == pdim.format(1 + i * 4)
+    assert var['foo1'].dimensions[2] == pdim.format(2 + i * 4)
+    assert var['foo2'].dimensions[0] == pdim.format(3 + i * 4)
+    assert var['foo2'].dimensions[1] == pdim.format(0 + i * 4)
+    assert var['foo2'].dimensions[2] == pdim.format(1 + i * 4)
+    assert var['foo3'].dimensions[0] == pdim.format(i * 4)
+    assert var['foo3'].dimensions[1] == pdim.format(1 + i * 4)
+    assert var['foo3'].dimensions[2] == pdim.format(2 + i * 4)
+    assert var['foo4'].dimensions[0] == pdim.format(3 + i * 4)
+    assert var['foo4'].dimensions[1] == pdim.format(i * 4)
+    assert var['foo4'].dimensions[2] == pdim.format(1 + i * 4)
+    assert var['x'].dimensions[0] == pdim.format(i * 4)
+    assert var['y'].dimensions[0] == pdim.format(i * 4)
+    assert var['z'].dimensions[0] == pdim.format(i * 4)
+    assert var['x1'].dimensions[0] == pdim.format(3 + i * 4)
+    assert var['y1'].dimensions[0] == pdim.format(i * 4)
+    assert var['z1'].dimensions[0] == pdim.format(i * 4)
+
+
 def test_invalid_netcdf4(tmp_local_or_remote_netcdf):
     h5 = get_hdf5_module(tmp_local_or_remote_netcdf)
-    with h5.File(tmp_local_or_remote_netcdf) as f:
-        f.create_dataset('foo', data=np.arange(5))
-        # labeled dimensions but no dimension scales
-        f['foo'].dims[0].label = 'x'
+    with h5.File(tmp_local_or_remote_netcdf, 'w') as f:
+        var, var2 = create_invalid_netcdf_data()
+        grps = ['bar', 'baz']
+        for grp in grps:
+            fx = f.create_group(grp)
+            for k, v in var.items():
+                fx.create_dataset(k, data=v)
+            for k, v in var2.items():
+                fx.create_dataset(k, data=np.arange(v))
+
+    with h5netcdf.File(tmp_local_or_remote_netcdf, 'r',
+                       phony_dims='sort') as dsr:
+        i = len(grps) - 1
+        for grp in grps[::-1]:
+            var = dsr[grp].variables
+            check_invalid_netcdf4(var, i)
+            i -= 1
+
+    with h5netcdf.File(tmp_local_or_remote_netcdf, 'r',
+                       phony_dims='access') as dsr:
+        for i, grp in enumerate(grps[::-1]):
+            print(dsr[grp])
+            var = dsr[grp].variables
+            check_invalid_netcdf4(var, i)
+
+    with netCDF4.Dataset(tmp_local_or_remote_netcdf, mode='r') as dsr:
+        for i, grp in enumerate(grps):
+            print(dsr[grp])
+            var = dsr[grp].variables
+            check_invalid_netcdf4(var, i)
+
     with h5netcdf.File(tmp_local_or_remote_netcdf, 'r') as ds:
         with raises(ValueError):
-            ds.variables['foo'].dimensions
+            ds['bar'].variables['foo1'].dimensions
+
+    with raises(ValueError):
+        with h5netcdf.File(tmp_local_or_remote_netcdf, 'r',
+                           phony_dims='srt') as ds:
+            pass
+
+
+def check_invalid_netcdf4_mixed(var, i):
+    pdim = 'phony_dim_{}'.format(i)
+    assert var['foo1'].dimensions[0] == 'y1'
+    assert var['foo1'].dimensions[1] == 'z1'
+    assert var['foo1'].dimensions[2] == pdim
+    assert var['foo2'].dimensions[0] == 'x1'
+    assert var['foo2'].dimensions[1] == 'y1'
+    assert var['foo2'].dimensions[2] == 'z1'
+    assert var['foo3'].dimensions[0] == 'y1'
+    assert var['foo3'].dimensions[1] == 'z1'
+    assert var['foo3'].dimensions[2] == pdim
+    assert var['foo4'].dimensions[0] == 'x1'
+    assert var['foo4'].dimensions[1] == 'y1'
+    assert var['foo4'].dimensions[2] == 'z1'
+    assert var['x'].dimensions[0] == 'y1'
+    assert var['y'].dimensions[0] == 'y1'
+    assert var['z'].dimensions[0] == 'y1'
+    assert var['x1'].dimensions[0] == 'x1'
+    assert var['y1'].dimensions[0] == 'y1'
+    assert var['z1'].dimensions[0] == 'z1'
+
+
+def test_invalid_netcdf4_mixed(tmp_local_or_remote_netcdf):
+    h5 = get_hdf5_module(tmp_local_or_remote_netcdf)
+    with h5.File(tmp_local_or_remote_netcdf, 'w') as f:
+        var, var2 = create_invalid_netcdf_data()
+        for k, v in var.items():
+            f.create_dataset(k, data=v)
+        for k, v in var2.items():
+            f.create_dataset(k, data=np.arange(v))
+
+        if h5py.__version__ < LooseVersion('2.10.0'):
+            f['foo2'].dims.create_scale(f['x1'])
+            f['foo2'].dims.create_scale(f['y1'])
+            f['foo2'].dims.create_scale(f['z1'])
+        else:
+            f['x1'].make_scale()
+            f['y1'].make_scale()
+            f['z1'].make_scale()
+        f['foo2'].dims[0].attach_scale(f['x1'])
+        f['foo2'].dims[1].attach_scale(f['y1'])
+        f['foo2'].dims[2].attach_scale(f['z1'])
+
+    with h5netcdf.File(tmp_local_or_remote_netcdf, 'r',
+                       phony_dims='sort') as ds:
+        var = ds.variables
+        check_invalid_netcdf4_mixed(var, 3)
+
+    with h5netcdf.File(tmp_local_or_remote_netcdf, 'r',
+                       phony_dims='access') as ds:
+        var = ds.variables
+        check_invalid_netcdf4_mixed(var, 0)
+
+    with netCDF4.Dataset(tmp_local_or_remote_netcdf, 'r') as ds:
+        var = ds.variables
+        check_invalid_netcdf4_mixed(var, 3)
+
+    with h5netcdf.File(tmp_local_or_remote_netcdf, 'r') as ds:
+        with raises(ValueError):
+            ds.variables['foo1'].dimensions
+
+
+def test_invalid_netcdf_malformed_dimension_scales(tmp_local_or_remote_netcdf):
+    h5 = get_hdf5_module(tmp_local_or_remote_netcdf)
+    with h5.File(tmp_local_or_remote_netcdf, 'w') as f:
+        foo_data = np.arange(125).reshape(5, 5, 5)
+        f.create_dataset('foo1', data=foo_data)
+        f.create_dataset('x', data=np.arange(5))
+        f.create_dataset('y', data=np.arange(5))
+        f.create_dataset('z', data=np.arange(5))
+
+        if h5py.__version__ < LooseVersion('2.10.0'):
+            f['foo1'].dims.create_scale(f['x'])
+            f['foo1'].dims.create_scale(f['y'])
+            f['foo1'].dims.create_scale(f['z'])
+        else:
+            f['x'].make_scale()
+            f['y'].make_scale()
+            f['z'].make_scale()
+        f['foo1'].dims[0].attach_scale(f['x'])
+
+    with raises(ValueError):
+        with h5netcdf.File(tmp_local_or_remote_netcdf, 'r',
+                           phony_dims='sort') as ds:
+            pass
 
 
 def test_hierarchical_access_auto_create(tmp_local_or_remote_netcdf):
@@ -540,7 +694,7 @@ def test_hierarchical_access_auto_create(tmp_local_or_remote_netcdf):
     ds.close()
 
 
-def test_netcdf4Dimid(tmp_local_netcdf):
+def test_Netcdf4Dimid(tmp_local_netcdf):
     # regression test for https://github.com/shoyer/h5netcdf/issues/53
     with h5netcdf.File(tmp_local_netcdf, 'w') as f:
         f.dimensions['x'] = 1
@@ -548,10 +702,12 @@ def test_netcdf4Dimid(tmp_local_netcdf):
         g.dimensions['x'] = 2
         g.dimensions['y'] = 3
 
-    with h5py.File(tmp_local_netcdf) as f:
-        assert f['x'].attrs['_Netcdf4Dimid'] == 0
-        assert f['foo/x'].attrs['_Netcdf4Dimid'] == 1
-        assert f['foo/y'].attrs['_Netcdf4Dimid'] == 2
+    with h5py.File(tmp_local_netcdf, 'r') as f:
+        # all dimension IDs should be present exactly once
+        dim_ids = {
+            f[name].attrs['_Netcdf4Dimid'] for name in ['x', 'foo/x', 'foo/y']
+        }
+        assert dim_ids == {0, 1, 2}
 
 
 def test_reading_str_array_from_netCDF4(tmp_local_netcdf):
@@ -608,24 +764,24 @@ def test_create_variable_matching_saved_dimension(tmp_local_or_remote_netcdf):
     if h5 is not h5py:
         pytest.xfail('https://github.com/shoyer/h5netcdf/issues/48')
 
-    with h5netcdf.File(tmp_local_or_remote_netcdf) as f:
+    with h5netcdf.File(tmp_local_or_remote_netcdf, 'w') as f:
         f.dimensions['x'] = 2
         f.create_variable('y', data=[1, 2], dimensions=('x',))
 
-    with h5.File(tmp_local_or_remote_netcdf) as f:
+    with h5.File(tmp_local_or_remote_netcdf, 'r') as f:
         assert f['y'].dims[0].keys() == [NOT_A_VARIABLE.decode('ascii')]
 
-    with h5netcdf.File(tmp_local_or_remote_netcdf) as f:
+    with h5netcdf.File(tmp_local_or_remote_netcdf, 'a') as f:
         f.create_variable('x', data=[0, 1], dimensions=('x',))
 
-    with h5.File(tmp_local_or_remote_netcdf) as f:
+    with h5.File(tmp_local_or_remote_netcdf, 'r') as f:
         assert f['y'].dims[0].keys() == ['x']
 
 
 def test_invalid_netcdf_warns(tmp_local_or_remote_netcdf):
     if tmp_local_or_remote_netcdf.startswith(remote_h5):
         pytest.skip('h5pyd does not support NumPy complex dtype yet')
-    with h5netcdf.File(tmp_local_or_remote_netcdf) as f:
+    with h5netcdf.File(tmp_local_or_remote_netcdf, 'w') as f:
         # valid
         with pytest.warns(None) as record:
             f.create_variable('lzf_compressed', data=[1], dimensions=('x'),
@@ -640,7 +796,7 @@ def test_invalid_netcdf_warns(tmp_local_or_remote_netcdf):
             f.create_variable('scaleoffset', data=[1], dimensions=('x',),
                               scaleoffset=0)
     h5 = get_hdf5_module(tmp_local_or_remote_netcdf)
-    with h5.File(tmp_local_or_remote_netcdf) as f:
+    with h5.File(tmp_local_or_remote_netcdf, 'r') as f:
         assert '_NCProperties' not in f.attrs
 
 
@@ -663,36 +819,47 @@ def test_invalid_netcdf_error(tmp_local_or_remote_netcdf):
 def test_invalid_netcdf_okay(tmp_local_or_remote_netcdf):
     if tmp_local_or_remote_netcdf.startswith(remote_h5):
         pytest.skip('h5pyd does not support NumPy complex dtype yet')
-    with h5netcdf.File(tmp_local_or_remote_netcdf, invalid_netcdf=True) as f:
+    with h5netcdf.File(tmp_local_or_remote_netcdf, 'w',
+                       invalid_netcdf=True) as f:
         f.create_variable('lzf_compressed', data=[1], dimensions=('x'),
                           compression='lzf')
         f.create_variable('complex', data=1j)
         f.attrs['complex_attr'] = 1j
         f.create_variable('scaleoffset', data=[1], dimensions=('x',),
                           scaleoffset=0)
-    with h5netcdf.File(tmp_local_or_remote_netcdf) as f:
+    with h5netcdf.File(tmp_local_or_remote_netcdf, 'r') as f:
         np.testing.assert_equal(f['lzf_compressed'][:], [1])
         assert f['complex'][...] == 1j
         assert f.attrs['complex_attr'] == 1j
         np.testing.assert_equal(f['scaleoffset'][:], [1])
     h5 = get_hdf5_module(tmp_local_or_remote_netcdf)
-    with h5.File(tmp_local_or_remote_netcdf) as f:
+    with h5.File(tmp_local_or_remote_netcdf, 'r') as f:
         assert '_NCProperties' not in f.attrs
 
 
+def test_reopen_file_different_dimension_sizes(tmp_local_netcdf):
+    # regression test for https://github.com/shoyer/h5netcdf/issues/55
+    with h5netcdf.File(tmp_local_netcdf, mode='w') as f:
+        f.create_variable('/one/foo', data=[1], dimensions=('x',))
+    with h5netcdf.File(tmp_local_netcdf, mode='a') as f:
+        f.create_variable('/two/foo', data=[1, 2], dimensions=('x',))
+    with netCDF4.Dataset(tmp_local_netcdf, mode='r') as f:
+        assert f.groups['one'].variables['foo'][...].shape == (1,)
+
+
 def test_invalid_then_valid_no_ncproperties(tmp_local_or_remote_netcdf):
-    with h5netcdf.File(tmp_local_or_remote_netcdf, invalid_netcdf=True):
+    with h5netcdf.File(tmp_local_or_remote_netcdf, 'w', invalid_netcdf=True):
         pass
-    with h5netcdf.File(tmp_local_or_remote_netcdf):
+    with h5netcdf.File(tmp_local_or_remote_netcdf, 'r'):
         pass
     h5 = get_hdf5_module(tmp_local_or_remote_netcdf)
-    with h5.File(tmp_local_or_remote_netcdf) as f:
+    with h5.File(tmp_local_or_remote_netcdf, 'r') as f:
         # still not a valid netcdf file
         assert '_NCProperties' not in f.attrs
 
 
 def test_creating_and_resizing_unlimited_dimensions(tmp_local_or_remote_netcdf):
-    with h5netcdf.File(tmp_local_or_remote_netcdf) as f:
+    with h5netcdf.File(tmp_local_or_remote_netcdf, 'w') as f:
         f.dimensions['x'] = None
         f.dimensions['y'] = 15
         f.dimensions['z'] = None
@@ -705,7 +872,7 @@ def test_creating_and_resizing_unlimited_dimensions(tmp_local_or_remote_netcdf):
 
     h5 = get_hdf5_module(tmp_local_or_remote_netcdf)
     # Assert some behavior observed by using the C netCDF bindings.
-    with h5.File(tmp_local_or_remote_netcdf) as f:
+    with h5.File(tmp_local_or_remote_netcdf, 'r') as f:
         assert f["x"].shape == (0,)
         assert f["x"].maxshape == (None,)
         assert f["y"].shape == (15,)
@@ -868,3 +1035,12 @@ def test_reading_unused_unlimited_dimension(tmp_local_or_remote_netcdf):
         assert f.dimensions == {'x': None}
 
     f = h5netcdf.File(tmp_local_or_remote_netcdf, 'r')
+
+
+def test_reading_special_datatype_created_with_c_api(tmp_local_netcdf):
+    "Test reading a file with unsupported Datatype"
+    with netCDF4.Dataset(tmp_local_netcdf, "w") as f:
+        complex128 = np.dtype([('real', np.float64), ('imag', np.float64)])
+        f.createCompoundType(complex128, 'complex128')
+    with h5netcdf.File(tmp_local_netcdf) as f:
+        pass
